package MT::Plugin::Moober;
use strict;
use MT;
use MT::Plugin;
use MT::Entry;
use File::Basename;
use Image::Size;
use MT::Image;
use MT::WeblogPublisher;
use MT::Util qw( offset_time_list format_ts );

our $VERSION = '0.0.1';

use base qw( MT::Plugin );

@MT::Plugin::Moober::ISA = qw(MT::Plugin);

my $plugin = new MT::Plugin::Moober({
    id => 'Moober',
    key => 'moober',
    description => '<MT_TRANS phrase=\'_PLUGIN_DESCRIPTION\'>',
    name => 'Moober',
    author_name => 'Junnama Noda',
    author_link => 'http://junnama.alfasado.net/online/',
    'version' => $VERSION,
    l10n_class => 'Moober::L10N',
    settings => new MT::PluginSettings([
        ['postfrom', { Default => '' }],
        ['account', { Default => '' }],
        ['passwd', { Default => '' }],
        ['server', { Default => '' }],
        ['tmpdir', { Default => '' }],
        ['outdir', { Default => '' }],
        ['protocol', { Default => 'pop3' }],
        ['db_charset', { Default => 'utf8' }],
        ['thumb_x', { Default => 200 }],
        ['post_blog_id', { Default => 1 }],
        ['author_id', { Default => 1 }],
        ['thumb_alt', { Default => 'Click for larger view' }],
        ['status', { Default => 2 }],
        ['rebuild', { Default => 1 }],
        ['imagepos', { Default => 1 }],
    ]),
    system_config_template => 'moober_config.tmpl',
});

MT->add_plugin($plugin);

sub init_registry {
    my $plugin = shift;
    $plugin->registry({
        tasks => {
            'Moober' => {
                label     => $plugin->translate('Post entry from e-mail'),
                frequency => 3,
                code      => \&_entry_from_email,
            },
        },
	    tags => {
            modifier => {
                'assetelement' => \&_assetelement,
            },
        }
   });
}

our %mime_type = (
    'gif'   =>  'image/gif',
    'jpeg'  =>  'image/jpeg',
    'jpg'   =>  'image/jpeg',
    'jpe'   =>  'image/jpeg',
    'png'   =>  'image/png',
);

sub _assetelement {
	my($text, $arg, $ctx) = @_;
	my @args = split(/\,/,$arg);
	my $element = $args[0];
	my $class = $args[1];
	my $closer = '';
	if ($element) {
        $closer = "</$element>";
	    $element = '<' . $element;
	    if ($class) {
	        $element .= ' class="' . $class . '"';
	    }
	    $element .= '>';	
	} else {
	    $element = '';
	}
	my $pattern = quotemeta('<span class="mt-enclosure');
	$text =~ s/$pattern.*?>(<.*?>)<\/span>/$element$1$closer/g;
	return $text;
}

sub _entry_from_email {
    my $app = MT->new;
    use Net::POP3;
    use MIME::Parser;
    use Encode;
    use Encode qw/encode decode/;
    my $blog_id = $plugin->get_config_value('post_blog_id');
    my $blog;
    if ($blog_id) {
        $blog = MT::Blog->load($blog_id);
    } else {
        $blog = MT::Blog->load( undef, { limit => 1 } );
        $blog_id = $blog->id;
    }
    my $author_id = $plugin->get_config_value('author_id');
    unless ($author_id) {
        my $author = MT::Author->load( undef, { limit => 1 } );
        $author_id = $author->id;
    }
    my $separator = '/';
    if ($^O eq 'MSWin32') { $separator = '\\' }
    my $outdir = $plugin->get_config_value('outdir');
    my $tmpdir = $plugin->get_config_value('tmpdir');
    if ($outdir =~ /(^.*)$separator$/) {
        $outdir = $1;
    }
    if ($tmpdir =~ /(^.*)$separator$/) {
        $tmpdir = $1;
    }
    my $postfrom = $plugin->get_config_value('postfrom');
    my $server = $plugin->get_config_value('server');
    my $account = $plugin->get_config_value('account');
    my $passwd = $plugin->get_config_value('passwd');
    my $protocol = $plugin->get_config_value('protocol');
    my $db_charset = $plugin->get_config_value('db_charset');
    my $thumb_alt = $plugin->get_config_value('thumb_alt');
    my $thumb_x = $plugin->get_config_value('thumb_x');
    my $status = $plugin->get_config_value('status');
    my $rebuild = $plugin->get_config_value('rebuild');
    my $imagepos = $plugin->get_config_value('imagepos');
    $thumb_x = int( $thumb_x );
    mkdir($outdir, 0777) unless (-d $outdir);
    mkdir($tmpdir, 0777) unless (-d $tmpdir);
    my $pop3  = Net::POP3->new($server) or die "Could not open pop3.";
    my $login = (lc($protocol) eq 'apop') ? 'apop' : 'login';
    my $count = $pop3->$login($account, $passwd);
    my $messages = $pop3->list();
    my @entries;
    foreach my $id ( sort ( keys %{$messages} ) ) {
        my $message = $pop3->get($id);
        my $parser = new MIME::Parser;
        $parser->output_dir($tmpdir);
        my $entity = $parser->parse_data($message);
        my $header = $entity->head;
        my $from = $header->get('From');
        my $subject = $header->get('Subject');
        $subject = encode($db_charset, decode('MIME-Header', $subject));
		$from = encode($db_charset, decode('MIME-Header', $from));
		$from =~ s/\n//sg; $subject =~ s/\n//sg;
		if ($from =~ /<(.{1,})>/) {
			$from = $1;
		}
		if ($from ne $postfrom) {
		    $pop3->delete($id);
		    next;
		}
        my $body; my @images;
		if ( $entity->is_multipart ) {
			opendir(DIR, $tmpdir);
			my @file = readdir(DIR);
			closedir(DIR);
			foreach my $f_name ( @file ) {
				unless ($f_name =~ /^\./) {
					my $f_pth = File::Spec->catfile ( $tmpdir, $f_name );
                    if ($f_name =~ /^msg\-[0-9]{1,}\-[0-9]{1,}\.txt/) {
                        open(FH, $f_pth);
                        while (<FH>) {
                            $body .= $_;
                        }
                        close(FH);
                        unlink($f_pth);
                        $body = encode( $db_charset, decode('iso-2022-jp', $body) );
                    } else {
                        my ( $globe_x, $globe_y ) = imgsize($f_pth);
                        unless ( defined $globe_x ) {
                            $globe_x = 0; # Why "Use of uninitialized value in concatenation ..."
                        }
                        my @suffix = split(/\./, $f_pth);
                        my $ext = lc( pop(@suffix) );
                        my $o_pth = File::Spec->catfile ( $outdir, $f_name );
                        $o_pth = &_uniq_fileneme( $o_pth );
                        rename ($f_pth, $o_pth);
                        my %asset_elements = (
                            'parent' => undef,
                            'file' => $o_pth,
                            'blog_id' => $blog_id,
                            'label' => '',
                            'description' => '',
                            'author_id' => $author_id,
                        );
                        my ($asset_id, $asset) = &_save_asset(%asset_elements);
                        my $tag = '<form mt:asset-id="' . $asset_id . '" class="mt-enclosure mt-enclosure-image">';
                        if ( $globe_x > $thumb_x ) {
                            my $t_pth = &_uniq_fileneme( $o_pth );
                            my $img = MT::Image->new( Filename => $o_pth );
                            my ( $blob, $w, $h ) = $img->scale( Width => $thumb_x );
                            open FH, ">$t_pth" || die "Can't create $t_pth!";
                            binmode FH;
                            print FH $blob;
                            close FH;
                            my ( $thumb_w, $thumb_h ) = imgsize($t_pth);
                            my $n_pth = $t_pth;
                            $n_pth =~ s/(^.*)\..*$/$1.'-thumb-'.$thumb_w.'x'.$thumb_h.'.'.$ext/e;
                            rename( $t_pth, $n_pth );
                            %asset_elements = (
                                'parent' => $o_pth,
                                'file' => $n_pth,
                                'blog_id' => $blog_id,
                                'label' => '',
                                'description' => '',
                                'author_id' => $author_id,
                            );
                            my ($t_asset_id, $t_asset) = &_save_asset(%asset_elements);
                            $tag .= '<a href="' . $asset . '"><img src="' . $t_asset . '" width="' . $thumb_w . '" height="'
                                                            . $thumb_h . '" alt="' . $thumb_alt . '" /></a></form>';
                        } else {
                            $tag .= '<img src="' . $asset . '" width="' . $globe_x . '" height="'
                                                                                . $globe_y . '" alt="" /></form>';
                        }
                        push ( @images, $tag );
					}
				}
			}
		} else {
			$body = $entity->bodyhandle;
			$body = $body->as_string;
			$body = encode( $db_charset, decode('iso-2022-jp', $body) );
			opendir(DIR, $tmpdir);
			my @file = readdir(DIR);
			closedir(DIR);
			foreach my $f_name (@file) {
				my $f_pth = $tmpdir . $separator . $f_name;
				unlink($f_pth);
			}
		}
		my $tags = '';
		if ( @images ) {
		    for my $tag ( @images ) {
		        $tags .= "$tag\n";
		    }
		}
		if ( $imagepos ) {
		    $body = $tags . $body;
		} else {
		    $body .= $tags;
		}
		my $entry = MT::Entry->new;
		$entry->blog_id($blog_id);
		$entry->author_id($author_id);
		$entry->status($status);
		$entry->class('entry');
		$entry->allow_comments($blog->allow_comments_default);
		$entry->allow_pings($blog->allow_pings_default);
		$entry->title($subject);
		$entry->text($body);
		$entry->save
		    or return $plugin->translate('Error saving entry: [_1]', $entry->errstr);
		push ( @entries, $entry );
		$pop3->delete($id);
	}
	$pop3->quit;
	if ( ( $rebuild ) && ( $status == 2 ) ) {
        if ( @entries ) {
            my $pub = MT::WeblogPublisher->new();
            for my $entry (@entries) {
                $pub->rebuild_entry( Entry => $entry,
                                     BuildDependencies => 1,
                                     NoIndexes => 1 )
                    or $app->error($plugin->translate('Rebuild error: [_1]', $pub->errstr));
            }
            $pub->rebuild_indexes( Blog => $blog ) 
                    or $app->error($plugin->translate('Rebuild error: [_1]', $pub->errstr));
        }
    }
	return 1;
}

sub _save_asset {
    my %asset_elements = @_;
    my $file_path = $asset_elements{'file'};
    if ($^O eq 'MSWin32') {
        $file_path =~ s/\//\\/;
    }
    my $parent = $asset_elements{'parent'};
    if ( ($^O eq 'MSWin32') && ($parent) ) {
        $parent =~ s/\//\\/;
    }
    my $author_id = $asset_elements{'author_id'};
    my $blog_id = $asset_elements{'blog_id'};
    my $label = $asset_elements{'label'};
    my $description = $asset_elements{'description'};
    my $basename = basename($file_path);
    my @suffix = split(/\./, $file_path);
    my $ext = pop(@suffix);
    $ext = lc($ext);
    my $mime_type = $mime_type{$ext};
    my $asset_class = 'file';
    if (($ext =~ /^jpe?g$/i) || ($ext =~ /^gif$/i) || ($ext =~ /^png$/i)) {
        $asset_class = 'image';
    }
    unless ($author_id) {
        my $author = MT::Author->load();
        $author_id = $author->id;
    }
    my $separator = '\/';
    if ($^O eq 'MSWin32') {
        $separator = '\\';
    }
    my $blog; my $asset_parant_id;
    unless ($blog_id) {
        my @blogs = MT::Blog->load();
        for my $item (@blogs) {
            my $check_path = $item->site_path;
            $check_path = quotemeta($check_path);
            if ($file_path =~ /^$check_path/) {
                $blog = $item;
                last;
            }
        }
    } else {
        $blog = MT::Blog->load( { id => $blog_id } );
    }
    $blog_id = $blog->id;
    my $site_path = $blog->site_path;
    if ($site_path =~ /(^.*)$separator$/) {
        $site_path = $1;
    }
    my $site_url = $blog->site_url;
    if ($site_url =~ /(^.*)$separator$/) {
        $site_url = $1;
    }
    $site_path = quotemeta($site_path);
    my $asset_path = $file_path;
    $asset_path =~ s/$site_path/%r/;
    my $asset_url = $asset_path;
    if ($^O eq 'MSWin32') {
        $asset_url =~ s/\\/\//g;
    }
    if ( $parent ) {
        my $parent_basename = basename($parent);
        $parent =~ s/$site_path/%r/;
        my $parent_asset_pkg = MT::Asset->handler_for_file($parent_basename);
        if ($asset_class eq 'image') {
            $parent_asset_pkg->isa('MT::Asset::Image');
        }
        my $asset_parant = $parent_asset_pkg->load( { file_path => $parent } );
        if (defined $asset_parant) {
            if ($asset_parant->blog_id != $blog_id) {
                $blog_id = $asset_parant->blog_id;
                $blog = MT::Blog->load( { id => $blog_id } );
                $site_path = $blog->site_path;
                if ($site_path =~ /(^.*)$separator$/) {
                    $site_path = $1;
                }
                $site_path = quotemeta($site_path);
                my $asset_path = $file_path;
                $asset_path =~ s/$site_path/%r/;
            }
            $asset_parant_id = $asset_parant->id;
            unless ($label) {
                $label = $asset_parant->label;
            }
            unless ($description) {
                $description = $asset_parant->description;
            }
        }
    }
    require MT::Asset;
    my $asset_pkg = MT::Asset->handler_for_file($basename);
    my $asset;
    if ($asset_class eq 'image') {
        $asset_pkg->isa('MT::Asset::Image');
    }
    if (!( $asset = $asset_pkg->load( { file_path => $asset_path, blog_id => $blog_id } ))) {
        $asset = $asset_pkg->new();
    }
    unless (defined $asset) {
        $asset = MT::Asset->load( { file_path => $asset_path } );
        if (defined $asset) {
        } else {
            $asset = MT::Asset->new;
        }
    }
    $asset->blog_id($blog_id);
    $asset->file_path($asset_path);
    $asset->url($asset_url);
    $asset->file_name($basename);
    $asset->mime_type($mime_type);
    $asset->file_ext($ext);
    $asset->class($asset_class);
    $asset->created_by($author_id);
    $asset->modified_by($author_id);
    if ($asset_parant_id) {
        $asset->parent($asset_parant_id);
    }
    my ($w,$h);
    if ($asset_class eq 'image') {
        ($w,$h) = imgsize($file_path);
        $asset->image_width($w);
        $asset->image_height($h);
    }
    if ($label) {
        $asset->label($label);
    }
    if ($description) {
        $asset->description($description);
    }
    $asset->save
        or return $plugin->translate('Error saving asset: [_1]', $asset->errstr);
    $asset_url =~ s/%r/$site_url/;
    return ($asset->id, $asset_url);
}

sub _uniq_fileneme {
    my $file = shift;
    unless (-f $file) {
        return $file;
    }
    my @suffix = split(/\./,$file);
    my $ext = pop(@suffix);
    my $base = $file;
    $base =~ s/(.{1,})\.$ext$/$1/;
    my $i = 0;
    do {
            $i++;
            $file = $base.'_'.$i.'.'.$ext;
        } while (-e $file);
    return $file;
}

1;